==> lino-23.3.2.tar.gz: lino-23.3.2/lino/utils/ranges.py <==
# -*- coding: UTF-8 -*-
# Copyright 2009-2021 Luc Saffre
# License: GNU Affero General Public License v3 (see file COPYING for details)

"""The :mod:`lino.utils.ranges` module contains utility methods for
working with "ranges".

A range is basically a tuple of two values `a` and `b`, called also
limits, who indicate the start and the end of the range.

An open range is a range that has at least one limit set to `None`.



"""


def constrain(value, lowest, highest):
    """Test whether `value` is within the range `(highest, lowest)`.
    Return `value` if it is inside the range, otherwise return
    `highest` if the value is *above* the range or `lowest` it the value
    is *below* the range.

    Examples in :doc:`/topics/utils`.

    """
    return min(highest, max(value, lowest))


def encompass(a, b):
    """
    Test whether range `a` encompasses (is wider than) range `b`.

    Examples in :doc:`/topics/utils`.
    """
    if a[0] is None:
        if a[1] is None:
            return True
        if b[1] is None:
            return False
        return (b[1] <= a[1])
    else:
        if b[0] is None or b[0] < a[0]:
            return False
        if a[1] is None:
            return True
        if b[1] is None:
            return False
        return a[1] >= b[1]


def isrange(a, b):
    """
    Return True if the passed tuple `(a,b)` is a valid range
    (that is, `a` may not be greater than `b`).
    """
    #~ if r[0] is None or r[1] is None: return True
    #~ if r[0] <= r[1]: return True
    if a is None or b is None:
        return True
    if a <= b:
        return True
    return False


def overlap2(a, b):
    """
    Same as :func:`overlap` but with different signature.
    """
    return overlap(a[0], a[1], b[0], b[1])


def overlap(a1, a2, b1, b2):
    """Test whether two value ranges overlap.

    This function is typically used with date values, but it also
    works with integers or other comparable values.  The following
    examples use integer values to be more readable.

    Unlike the test presented at
    <http://bytes.com/topic/python/answers/457949-determing-whether-two-ranges-overlap>,
    this works also with "open" ranges (the open end being indicated
    by a `None` value).

    Types of constellations::

      -   o---o  o---o
      -   o---o  o--->
      -   <---o  o---o
      -   <---o  o--->

      -   o------------->
                o---o

      -   o---o
            o---o

      -   o---o
            o--->
      -   <---------o
               o---o

    NB: Ranges that "only touch" each other are not considered overlapping.

    Examples in :doc:`/topics/utils`.

    """

    #~ return a2 > b1 and a1 < b2

    if a2:
        if b1:
            if b1 >= a2:
                return False
            else:
                if b2 and a1:
                    if a1 > a2:
                        raise ValueError("Range 1 ends before it started.")
                    return b2 > a1
                else:
                    return True
        else:
            if b2 and a1:
                return b2 >= a1
            else:
                return True
    elif b2:
        if a1:
            return b2 > a1
        else:
            return True
    return True


def _test():
    import doctest
    doctest.testmod()

if __name__ == "__main__":
    _test()

==> lino-25.10.4.tar.gz: lino-25.10.4/lino/utils/ranges.py <==
# -*- coding: UTF-8 -*-
# Copyright 2009-2024 Rumma & Ko Ltd
# License: GNU Affero General Public License v3 (see file COPYING for details)
"""The :mod:`lino.utils.ranges` module contains utility methods for
working with "ranges".

A range is basically a tuple of two values `a` and `b`, called also
limits, who indicate the start and the end of the range.

An open range is a range that has at least one limit set to `None`.



"""


def constrain(value, lowest, highest):
    """Test whether `value` is within the range `(highest, lowest)`.
    Return `value` if it is inside the range, otherwise return
    `highest` if the value is *above* the range or `lowest` it the value
    is *below* the range.

    Examples in :doc:`/topics/utils`.

    """
    return min(highest, max(value, lowest))


def encompass(a, b):
    """
    Test whether range `a` encompasses (is wider than) range `b`.

    Examples in :doc:`/topics/utils`.
    """
    if a[0] is None:
        if a[1] is None:
            return True
        if b[1] is None:
            return False
        return b[1] <= a[1]
    else:
        if b[0] is None or b[0] < a[0]:
            return False
        if a[1] is None:
            return True
        if b[1] is None:
            return False
        return a[1] >= b[1]


def isrange(a, b):
    """
    Return True if the passed tuple `(a,b)` is a valid range
    (that is, `a` may not be greater than `b`).
    """
    # ~ if r[0] is None or r[1] is None: return True
    # ~ if r[0] <= r[1]: return True
    if a is None or b is None:
        return True
    if a <= b:
        return True
    return False


def overlap2(a, b):
    """
    Same as :func:`overlap` but with different signature.
    """
    return overlap(a[0], a[1], b[0], b[1])


def overlap(a1, a2, b1, b2):
    """Test whether two value ranges overlap.

    This function is typically used with date values, but it also
    works with integers or other comparable values.  The following
    examples use integer values to be more readable.

    Unlike the test presented in `Determing whether two ranges overlap
    <https://post.bytes.com/forum/topic/python/398735-determing-whether-two-ranges-overlap?t=457949>`__,
    this works also with "open" ranges (the open end being indicated by a `None`
    value).

    Types of constellations::

      -   o---o  o---o
      -   o---o  o--->
      -   <---o  o---o
      -   <---o  o--->

      -   o------------->
                o---o

      -   o---o
            o---o

      -   o---o
            o--->
      -   <---------o
               o---o

    NB: Ranges that "only touch" each other are not considered overlapping.

    Examples in :doc:`/topics/utils`.

    """

    # ~ return a2 > b1 and a1 < b2

    if a2:
        if b1:
            if b1 >= a2:
                return False
            else:
                if b2 and a1:
                    if a1 > a2:
                        raise ValueError("Range 1 ends before it started.")
                    return b2 > a1
                else:
                    return True
        else:
            if b2 and a1:
                return b2 >= a1
            else:
                return True
    elif b2:
        if a1:
            return b2 > a1
        else:
            return True
    return True
